package com.s_cradle.improve.client;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Command;

/**
 * Copyright (C) 2003 Sophia Cradle, Inc.
 *
 * Improve Ŏgj[̈ʓIȋ@\񋟂钊ۃNXłB
 *
 * @author Masaru Yoshimura
 */
public abstract class Menu {
    protected static final int LIST_BG_COLOR = 0x00DDDDFF;
    private static final int COLOR_FRAME     = 0x000000;
    private static final int COLOR_SHADOW    = 0x444444;
    private static final int LEFT_INTERVAL = 5;
    private static final int RIGHT_INTERVAL = 5;
    protected static final int COLUMN_HEIGHT = 17;
    protected static final int STRING_MARGIN_HEIGHT = 3;

    protected static final Command okCommand = new Command("OK",Command.OK,1);
    protected static final Command cancelCommand = new Command("ݾ",Command.CANCEL,1);

    private String title;
    private String[] items;
    protected int cursor;
    private int width;
    private int height;
    private boolean containsTitleWidth;

    /**
     * RXgN^
     */
    public Menu(String title_, String[] items_, boolean containsTitleWidth_) {
        cursor = 0;
        containsTitleWidth = containsTitleWidth_;
        if( title_ != null && items_ != null ){
            setTitleAndItems(title_, items_);
        }
    }

    /**
     * RXgN^
     */
    public Menu(String title_, String[] items_) {
        this(title_, items_, false);
    }

    /**
     * ^Cgƍڂ̐ݒ
     */
    public void setTitleAndItems(String title_, String[] items_){
        title = title_;
        items = items_;

        Font font = Font.getDefaultFont();
        width = (containsTitleWidth) ? font.stringWidth(title)+2 : 0;
        for(int i=0;i<items.length;i++){
            int temp = LEFT_INTERVAL+font.stringWidth(items[i])+RIGHT_INTERVAL;
            if(width< temp){
                width = temp;
            }
        }
        height = COLUMN_HEIGHT*(1+items.length)+1;
    }

    /**
     * L[nh
     */
    public Menu processKeyRelease(int gameAction, int keyCode){
        switch( gameAction ){
            case Canvas.FIRE:
                return processFireKeyRelease();
            case Canvas.DOWN:
                cursor++;
                if( cursor >= items.length ){
                    cursor --;
                }
                break;
            case Canvas.UP:
                cursor--;
                if( cursor < 0 ){
                    cursor ++;
                }
                break;
            default:
                break;
        }

        return this;
    }

    /**
     * L[nh
     */
    protected abstract Menu processFireKeyRelease();

    /**
     * `
     */
    public void draw(Graphics g, int width_, int height_){
        g.translate((width_-width)/2, (height_-height)/2);

        draw3DRect(g,0,0,width,height,LIST_BG_COLOR,0x00EEEEFF,0x00000080);
        drawFrameShadow(g,0,0,width,height,COLOR_FRAME, COLOR_SHADOW);

        g.setClip(1,1,width-2,height-2);

        g.setColor(0x00646464);
        g.fillRect(1, 1, width-2, COLUMN_HEIGHT);
        g.setColor(0x00CCCCCC);
        g.drawString(title,2,4, Graphics.LEFT | Graphics.TOP);

        for(int i=0;i<items.length;i++){
            if(cursor == i){
                g.setColor(0x0000FF);
                g.fillRect(1, COLUMN_HEIGHT*(i+1), width-2, COLUMN_HEIGHT);
                g.setColor(0xFFFFFF);
            } else{
                g.setColor(0x00000000);
            }
            g.drawString(items[i],LEFT_INTERVAL,COLUMN_HEIGHT*(i+1)+STRING_MARGIN_HEIGHT, Graphics.LEFT | Graphics.TOP);
        }
        g.translate(-g.getTranslateX(), -g.getTranslateY());
        g.setClip(0,0,width_,height_);
    }

    /**
     * Õt̋`̕`
     */
    protected static void draw3DRect(Graphics graph, int x,int y, int dx,int dy,
                                    int base,int light, int shadow){
        graph.setColor(base);
        graph.fillRect(x,y,dx,dy);
        graph.setColor(light);
        graph.drawLine(x,y,x + dx - 2,y);
        graph.drawLine(x,y,x,y + dy - 2);
        graph.setColor(shadow);
        graph.drawLine(x + dx - 1,y + 1,x + dx - 1,y + dy - 1);
        graph.drawLine(x + 1,y + dy - 1,x + dx - 1,y + dy - 1);
        return;
    }

    /**
     * e̕`
     */
    protected static void drawFrameShadow(Graphics g,int x, int y, int width,
            int height, int frame, int shadow)
    {
        g.setColor(frame);
        g.drawRect(x-1,y-1,width+1,height+1);
        g.setColor(shadow);
        g.drawLine(x,y + height + 1,x + width + 1,y + height + 1);
        g.drawLine(x + width + 1,y,x + width + 1,y + height + 1);
    }
}
